# -*- coding: utf8 -*-
from __future__ import unicode_literals

import six


from collections import OrderedDict
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimList,
    HprimMultiple,
    HprimString,
    Segment,
)


class DMSegmentH(HeaderSegment):
    sender_id = HprimString(5)
    sender_facility = HprimString(6)
    sender_telephone = HprimString(8)
    receiver_id = HprimString(10)
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1394-97")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class DMSegmentQ(Segment):
    specimen_id = HprimString((3, 2))
    test_id = HprimString(5, default="ALL")


class DMSegmentP(Segment):
    pid = HprimString(3)
    last_name = HprimString((6, 1), max_length=100)
    first_name = HprimString((6, 2), max_length=100)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    origin = HprimString(10, max_length=200)
    location = HprimString(26, max_length=100)
    orders = HprimForeign("O", "patient")


class DMSegmentO(Segment):
    specimen_id = HprimString(3, max_length=24)

    # Possible values hare WBC, RBC and PLT
    tests = HprimList(5, basefield=HprimMultiple(0, name=HprimString(2)))

    # R: Routine
    # S: Stat
    priority = HprimString(6)

    collection_date = HprimDate(8)

    # A: Add order
    # C: Cancel order
    action_code = HprimString(12)

    clinical_information = HprimString(14)

    attending_physician = HprimString(17, max_length=100)

    specific_parameters = HprimString(21)

    # x10e9/L Two decimal
    wbc_count = HprimString((22, 1))
    # x10e12/L. One decimal
    rbc_concentration = HprimString((22, 2))

    end_count = HprimString((22, 25), necessary=False)
    # F: sending results
    # X: Order is cancelled (The DM user has cancelled the order)
    # Q: Order information returned
    # Y: No order information found
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class DMSegmentR(Segment):
    analysis_type = HprimString((3, 2))
    test = HprimString((3, 4))
    value = HprimString(4)
    unit = HprimString(5)
    reference_ranges = HprimString(6)
    abnormal_flags = HprimList(7)
    status = HprimString(9, default="F")
    operator = HprimString((11, 2))
    date = HprimDate(13)


class DMSegmentC(Segment):
    # P: Practice
    source = HprimString(3, default="P")
    text = HprimString(4)
    # G: Generic
    comment_type = HprimString(5, default="G")


class DMSegmentL(Segment):
    # always empty
    code = HprimString(3)


class DMDriverBase(AstmMessageV2):
    DEFAULT_SEPARATORS = "|\\^&"
    ZERO_VALUES = ("0", "0.0", "")

    OPTIONS = [
        AstmMessageV2.Option(
            "nombre_cellules",
            label="Nombre de cellules à compter sur les WBC (par défaut)",
            default="100",
        ),
        AstmMessageV2.BooleanOption(
            "dm_ignorer_zero", label="Ignorer les valeurs à zéro", default=False
        ),
        AstmMessageV2.BooleanOption(
            "dm_wbcconfirm",
            label="Traitement du résultat de type WBCCONFIRM",
            default=True,
        ),
    ]

    def __init__(self, nombre_cellules="100", **kwargs):
        self.nombre_cellules = nombre_cellules
        self.code_WBC_RBC_PLT = ["WBC", "RBC", "PLT"]
        self.dm_ignorer_zero = kwargs.pop("dm_ignorer_zero", False)
        self.dm_wbcconfirm = kwargs.pop("dm_wbcconfirm", True)

    def create(self, **kwargs):
        message = self.SEGMENT_CLASS_MAP["H"](type_segment="H", **kwargs)
        message.end = self.SEGMENT_CLASS_MAP["L"](type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        tubes_ok = []
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: "A"},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                pid=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                location=patient.location,
            )
            for sample, action, priority in patient["samples"]:
                tests = list(
                    set(
                        [
                            o["code"]
                            for o in sample["orders"]
                            if o["code"] in self.code_WBC_RBC_PLT
                        ]
                    )
                )
                if not tests:
                    continue
                tubes_ok.append(sample.sample_id)
                physician = [
                    p for p in sample.physicians if p.type.lower() == "prescriber"
                ][0]
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    priority=priority,
                    collection_date=sample.collection_date,
                    action_code=action,
                    clinical_information=sample.diagnostic,
                    attending_physician=physician.code if physician else "",
                    report_type="Q",
                )
                nb_cellules = self.nombre_cellules
                for result in sample["results"]:
                    if result["code"] == self.code_WBC_RBC_PLT[0]:
                        try:
                            if isinstance(result["value"], six.string_types):
                                seg_o.wbc_count = "%.2f" % (
                                    float(result["value"].replace(",", ".", 1)),
                                )
                            else:
                                seg_o.wbc_count = "%.2f" % (float(result["value"]),)
                        except ValueError:
                            pass
                    elif result["code"] == self.code_WBC_RBC_PLT[1]:
                        try:
                            if isinstance(result["value"], six.string_types):
                                seg_o.rbc_concentration = "%.1f" % (
                                    float(result["value"].replace(",", ".", 1)),
                                )
                            else:
                                seg_o.rbc_concentration = "%.1f" % (
                                    float(result["value"]),
                                )
                        except ValueError:
                            pass
                    elif result["code"] == "NB_CELLULES":
                        nb_cellules = result["value"]
                if tests:
                    tests = sorted(tests, key=self.code_WBC_RBC_PLT.index)
                    # Ici on envoie le nombre de cellules que si WBC est présent
                    for analysis, specific in (
                        ([self.code_WBC_RBC_PLT[0]], "%s^" % (nb_cellules,)),
                        ([self.code_WBC_RBC_PLT[1]], "^"),
                        ([self.code_WBC_RBC_PLT[2]], "^"),
                        (self.code_WBC_RBC_PLT[:2], "%s^\\^" % (nb_cellules,)),
                        (self.code_WBC_RBC_PLT[1:3], "^\\^"),
                        (self.code_WBC_RBC_PLT, "%s^\\^\\^" % (nb_cellules,)),
                    ):
                        if tests == analysis:
                            seg_o.set_raw("specific_parameters", specific)
                    code_tests = {
                        self.code_WBC_RBC_PLT[0]: "WBC",
                        self.code_WBC_RBC_PLT[1]: "RBC",
                        self.code_WBC_RBC_PLT[2]: "PLT",
                    }
                    seg_o.set_raw(
                        "tests",
                        "\\".join(["^%s^^" % code_tests[code] for code in tests]),
                    )

        if context["query"] is not None:
            for tube in self.get_queries(context["query"]):
                if tube not in tubes_ok:
                    seg_p = msg.patients.add()
                    seg_p.orders.add(specimen_id=tube, report_type="Y")

        if len(msg.patients):
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(sample_id=seg_q.specimen_id, type="patient")
            return worklist
        worklist = Worklist(context="result")
        comment_dicts = OrderedDict(
            [
                ("Remarque reçue", {"code": "SIL", "results": []}),
                ("Remarques de l'utilisateur", {"code": "USER", "results": []}),
                ("Remarques sur la lame", {"code": "SLIDE", "results": []}),
                ("Leucocytes", {"code": "WBC", "results": []}),
                ("Erythrocytes", {"code": "RBC", "results": []}),
                ("Plaquettes", {"code": "PLT", "results": []}),
                ("LO", {"code": "BF", "results": []}),
            ]
        )
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    type="patient", sample_id=seg_o.specimen_id
                )
                for seg_r in seg_o.results:
                    if not self.dm_ignorer_zero or seg_r.value not in self.ZERO_VALUES:
                        sample.results.add(
                            code=seg_r.test,
                            value=seg_r.value,
                            date=seg_r.date,
                            operator=seg_r.operator,
                        )
                for seg_c in seg_o.commentaires:
                    leCom = None
                    for i, c in enumerate(seg_c.text.split("&XD&&XA&")):
                        tc = c.split(":")
                        if len(tc) > 1:
                            motif = tc[0]
                            m = comment_dicts.get(motif, None)
                            if m:
                                leCom = m
                                if tc[1].strip():
                                    leCom["results"].append(tc[1].split("~")[0].strip())
                                continue
                        if leCom and c.split("~")[0].strip():
                            leCom["results"].append(c.split("~")[0].strip())

                    for key, com in comment_dicts.items():
                        if com["results"]:
                            sample.results.add(
                                code="COM_" + com["code"],
                                value=";".join(com["results"]),
                                date=seg_o.results[0].date,
                            )
                            for n, r in enumerate(com["results"]):
                                sample.results.add(
                                    code="COM_" + com["code"] + str(n + 1),
                                    value=r,
                                    date=seg_o.results[0].date,
                                )
        for tube in list(sample):
            tests = [r["code"] for r in sample.results]

            if (
                self.dm_wbcconfirm and "WBCCONFIRM/" not in tests and seg_o.results
            ):  # On garde les résultats initiaux
                for required in ("MO/%", "LY/%", "EO/%", "BA/%", "SNE/%"):
                    if required not in tests:
                        sample.results.add(code=required, value="0", date=None)
        return worklist
