# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import logging

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimMultiple
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment

logger = logging.getLogger("driverng.eurobio_m10")


class EurobioM10SegmentMSH(HeaderSegment):
    sending_application = HL7String(3)
    sending_facility = HL7String(4)
    receiving_application = HL7String(5)
    receiving_facility = HL7String(6)
    date_of_message = HL7Date(7)
    message_type = HprimMultiple(
        9,
        message_code=HL7String(1),
        trigger_event=HL7String(2),
        message_structure=HL7String(3),
    )
    message_control_id = HL7String(10)
    processing_id = HL7String(11)
    version = HL7String(12)
    accept_acknowlegment_type = HL7String(15)
    application_acknowlegment_type = HL7String(16)
    country_code = HL7String(17)
    character_set = HL7String(18)
    principal_language_of_message = HL7String(19)
    message_profile_identifier = HL7String(21)

    patients = HL7Foreign("PID", "header")


class EurobioM10SegmentPID(Segment):
    patient_identifier = HL7String(4)

    orders = HL7Foreign("OBR", "patient")


class EurobioM10SegmentOBR(Segment):
    placer_order_number = HL7String(3)
    filler_order_number = HL7String(4)
    universal_service_identifier = HL7String((5, 1))
    universal_service_identifier_text = HL7String((5, 2))
    name_of_coding_system = HL7String((5, 3))
    observation_date = HL7Date(8)
    observation_end_date = HL7Date(9)

    results = HL7Foreign("OBX", "order")
    infos = HL7Foreign("NTE", "order")


class EurobioM10SegmentOBX(Segment):
    value_type = HL7String(3)
    observation_identifier = HL7String((4, 1))
    observation_text = HL7String((4, 2))
    sample_type = HL7String((4, 3))
    name_of_coding_system = HL7String((4, 4))
    observation_sub_id = HL7String(5)
    value = HL7String(6)
    units = HL7String(7)
    reference_range = HL7String(8)
    abnormal_flags = HL7String(9)
    observation_result_status = HL7String(12)
    observation_date = HL7String(15)
    producer_id = HL7String(16)
    responsible_observer = HL7String(17)
    observation_method = HL7String(18)
    equipment_instance_identifier = HL7String(19)
    analysis_date = HL7Date(20)


class EurobioM10SegmentNTE(Segment):
    comment_source = HL7String(3)
    comment = HL7String(4)


class Driver(AstmMessageV2):
    SPEC = {"code": "eurobio_m10", "label": "EUROBIO M10 HL7"}
    SEGMENT_CLASS_MAP = {
        "MSH": EurobioM10SegmentMSH,
        "PID": EurobioM10SegmentPID,
        "OBR": EurobioM10SegmentOBR,
        "OBX": EurobioM10SegmentOBX,
        "NTE": EurobioM10SegmentNTE,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_pid in message.patients:
            for seg_obr in seg_pid.orders:
                control_check = False
                general_code = ""
                general_date = ""
                general_operator = ""
                for seg_obx in seg_obr.results:
                    if seg_obx.sample_type == "QC":
                        control_check = True
                        sample = worklist.get_or_create(
                            sample_id=seg_obr.universal_service_identifier,
                            control_lot=seg_obr.universal_service_identifier,
                            type="control",
                        )
                    elif not control_check:
                        sample = worklist.get_or_create(
                            sample_id=seg_pid.patient_identifier, type="patient"
                        )

                    if seg_obx.value_type == "TX":
                        general_code = seg_obx.observation_text
                        general_date = datetime.datetime.strptime(
                            seg_obx.observation_date[:15], "%Y%m%d%H:%M:%S"
                        )
                        general_operator = seg_obx.responsible_observer
                        sample.results.add(
                            code=general_code,
                            sub_code=seg_obx.observation_sub_id,
                            value=seg_obx.value,
                            date=general_date,
                            operator=general_operator,
                            flags=[
                                seg_obx.abnormal_flags,
                                seg_obx.observation_result_status,
                            ],
                        )
                    elif seg_obx.value_type == "NM":
                        if seg_obx.observation_identifier == "":
                            code = "{} Cycle Threshold".format(general_code)
                        else:
                            code = "{} {}".format(
                                general_code, seg_obx.observation_text
                            )
                        sample.results.add(
                            code=code,
                            sub_code=seg_obx.observation_sub_id,
                            value=seg_obx.value,
                            unit=seg_obx.units,
                            date=general_date,
                            operator=general_operator,
                            flags=[seg_obx.observation_result_status],
                        )

        return worklist
